program Test;
{$OPTIMISE}

{ $W+ Warnings}           {Warnings off}

Device = mega48, VCC=5;

Import SysTick;

Define
  ProcClock      = 8000000;        {Hertz}
  SysTick        = 20;             {msec}
  StackSize      = $0064, iData;
  FrameSize      = $0064, iData;

Implementation

{$IDATA}

{--------------------------------------------------------------}
{ Type Declarations }

type

{--------------------------------------------------------------}
{ Const Declarations }

{--------------------------------------------------------------}
{ Var Declarations }

{$EEPROM}

{$IDATA}
var   Led1[@PortB, 4] : bit;
      Dummy: byte;

{Declarationen fr interne Register}

{$DATA}
      Register8[$8]: byte;                 {R8 fr ISR}

{--------------------------------------------------------------}
{ Functions }

{$IDATA}

{**************************************************************}
{ Init_Ports: schaltet alle bentigten Ports                   }
{                                                              }
{**************************************************************}
procedure Init_Ports;
begin
  DDRB:= %00111110;
  DDRC:= %00000010;
  DDRD:= %11110000;
end;

{**************************************************************}
{ Enable_Ints: Aktiviert alle bentigten Interrupts            }
{                                                              }
{**************************************************************}
procedure Enable_Interrupts;
begin
end;

{**************************************************************}
{ Wait4Bit: Wartet auf ein empfangenen Bit                     }
{                                                              }
{**************************************************************}
procedure Wait4Bit;
begin
  asm;
    SBRS       R8, 6                       ; Weiter wenn Send_CU = 1
    RJMP       Wait4Bit_Continue
    SET                                    ; CU kann gesetzt werden (Merker, dass CU nur einmal an geht; wegen REPEAT Schleife
    BLD        R8, 7

Wait4Bit:
    SBIC       PinD, 2                     ; Weiter wenn Gleis1 = 0
    RJMP       Wait4Bit_Continue           ; Sonst weiter mit Wait4Bit
    SBIC       PinD, 3                     ; Weiter wenn Gleis2 = 0
    RJMP       Wait4Bit_Continue           ; Sonst weiter mit Wait4Bit

    SBRS       R8, 7                       ; Weiter wenn CU gesetzt werden kann
    RJMP       Wait4Bit_Continue

    SBI        PortD, 5                    ; CU an
    SBI        PortB, 4                    ; Led an
    CLT                                    ; CU ist jetzt an (Geht erst in INT0/INT1 wieder aus
    BLD        R8, 7

Wait4Bit_Continue:
    SBRS       R8, 1                       ; repeat
    RJMP       Wait4Bit                    ; until Bit_Change;

    MOV        R9, R1                      ; Save Last_State
    CLT                                    ; Bit_Change := false
    BLD        R8, 1
    BST        R8, 0                       ; Bit nach T

    BLD        R4, 7                       ; End_Proc ohne SET T
    RET
  endasm;
end;

{**************************************************************}
{ INT0: Wird gestartet, wenn steigende Flanke auf Gleis1       }
{                                                              }
{**************************************************************}
{$NOSAVE}
interrupt Int0;
begin
  asm;
    CBI        PortB, 4                    ; Led aus
    INC        R2                          ; INC(Edges)
    LDI        R16, 01h                    ; H erkannt
    MOV        R17, R1                     ; letzten Wert laden
    MOV        R1, R16                     ; neuen speichern
    EOR        R16, R17                    ; wenn ungleich 1, sonst 0
    BST        R16, 0                      ; Ergebnis nach T
    BLD        R8, 0                       ; T nach Detect_Bit

    SET                                    ; Bit_Change := true
    BLD        R8, 1                       ; **** Bis hier ca. 6us
  endasm;
end;

{**************************************************************}
{ INT1: Wird gestartet, wenn steigende Flanke auf Gleis2       }
{                                                              }
{**************************************************************}
{$NOSAVE}
interrupt Int1;
begin
  asm;
    CBI        PortB, 4                    ; Led aus
    INC        R2                          ; INC(Edges)
    LDI        R16, 00h                    ; H erkannt
    MOV        R17, R1                     ; letzten Wert laden
    MOV        R1, R16                     ; neuen speichern
    EOR        R16, R17                    ; wenn ungleich 1, sonst 0
    BST        R16, 0                      ; Ergebnis nach T
    BLD        R8, 0                       ; T nach Detect_Bit

    SET                                    ; Bit_Change := true
    BLD        R8, 1                       ; **** Bis hier ca. 6us
  endasm;
end;

{--------------------------------------------------------------}
{ Main Program }
{$IDATA}

begin
  Enable_Interrupts;                       {Ints an}
  Init_Ports;
  Dummy := 0;

  Register8 := 0;

  loop
  endloop;

end Test.




